#See http://www.cprover.org/cprover-manual/, in particular http://www.cprover.org/cprover-manual/cbmc/tutorial/ and http://www.cprover.org/cprover-manual/test-suite/

rm -f *log

#First example: verify if there are indexes out of bounds on some array on the stack
./cbmc 1.c --bounds-check > 1.bounds-check.log 2>&1
#the same program is checked with valgrind, which is not able to detect errors on arrays on the stack...
gcc -ggdb 1.c -o 1
valgrind --leak-check=full --show-leak-kinds=all --log-file=1.valgrind.1.log ./1 > 1.exec.1.log
#... not even when they actually pop up
valgrind --leak-check=full --show-leak-kinds=all --log-file=1.valgrind.2.log ./1 1 > 1.exec.2.log
#of course, directly executing the program is even worse
./1 1

#Second example: verify if there are indexes out of bounds on some array on the heap
./cbmc 2.c --pointer-check --bounds-check > 2.pointer-check.bounds-check.log 2>&1
#the same program is checked with valgrind, which is able to detect errors on arrays on the heap...
gcc -ggdb 2.c -o 2
#...but only if you actually run it with the actual input leading to the error! No correctness guarantee
valgrind --leak-check=full --show-leak-kinds=all --log-file=2.valgrind.1.log ./2 > 2.exec.1.log
valgrind --leak-check=full --show-leak-kinds=all --log-file=2.valgrind.2.log ./2 1 > 2.exec.2.log
#again, directly executing the program is even worse
./2 1

#Alas CBMC is not always able to check correctness: it may not be able to understand which is the bound for loops (actually, a bound may not exist...)
timeout 2 ./cbmc 3.c --function binsearch --bounds-check --unwinding-assertions > 3.function.binsearch.bounds-check.unwinding-assertions.log 2>&1
#... thus it is necessary to manually provide a bound with the --unwind parameter
./cbmc 3.c --function binsearch --unwind 6 --bounds-check --unwinding-assertions > 3.function.binsearch.unwind.6.bounds-check.unwinding-assertions.log 2>&1
#same program with an actual error
timeout 2 ./cbmc 3.err.c --function binsearch --bounds-check --unwinding-assertions > 3.err.function.binsearch.bounds-check.unwinding-assertions.log 2>&1
#... thus it is necessary to manually provide a bound with the --unwind parameter
./cbmc 3.err.c --function binsearch --unwind 6 --bounds-check --unwinding-assertions > 3.err.function.binsearch.unwind.6.bounds-check.unwinding-assertions.log 2>&1

#MC/DC coverage
./cbmc 4.c --cover mcdc --unwind 6 > 4.cover.mcdc.unwind.6.log 2>&1
